package com.saggitt.omega.icons

import android.content.Context
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import androidx.core.graphics.ColorUtils
import androidx.palette.graphics.Palette
import com.android.launcher3.icons.BuildConfig

val Context.prefs
    get() = applicationContext.getSharedPreferences(
        getPrefName(),
        Context.MODE_PRIVATE
    )!!

fun shouldWrapAdaptive(context: Context) = context.prefs.getBoolean("pref_adaptive_icon_pack", false)
fun coloredBackground(context: Context) = context.prefs.getBoolean("pref_colored_background", false)

fun getWrapperBackgroundColor(context: Context, icon: Drawable): Int {
    val lightness = context.prefs.getFloat("pref_coloredBackgroundLightness", 0.5f)
    val palette = Palette.Builder(drawableToBitmap(icon)).generate()
    val dominantColor = palette.getDominantColor(Color.WHITE)
    return setLightness(dominantColor, lightness)
}

private fun setLightness(color: Int, lightness: Float): Int {
    if (color == Color.WHITE) {
        return color
    }
    val outHsl = floatArrayOf(0f, 0f, 0f)
    ColorUtils.colorToHSL(color, outHsl)
    outHsl[2] = lightness
    return ColorUtils.HSLToColor(outHsl)
}

fun drawableToBitmap(drawable: Drawable): Bitmap {
    if (drawable is BitmapDrawable) {
        return drawable.bitmap
    }

    val width = drawable.intrinsicWidth.coerceAtLeast(1)
    val height = drawable.intrinsicHeight.coerceAtLeast(1)
    val bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
    val canvas = Canvas(bitmap)
    drawable.setBounds(0, 0, canvas.width, canvas.height)
    drawable.draw(canvas)
    return bitmap
}

private fun getPrefName(): String {
    return if (BuildConfig.BUILD_TYPE.contains("debug")) {
        "com.saggitt.omega.debug_preferences"
    } else if (BuildConfig.BUILD_TYPE.contains("neo")) {
        "com.saggitt.omega.neo_preferences"
    } else {
        "com.saggitt.omega_preferences"
    }
}